classdef CT_model<handle
    %CT_MODEL Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        JAsteel %
        S %
        w1 % . 
        w2 % . 
        l %  
        
        R2 % 
        Rb % 
        L2 % 
        Lb  %     
        
       
        
        Nstep=100;
        epsilon=0.01
    end
    
    methods
        function this=CT_model(JAsteel, S, w1, w2, l, R2, Rb, L2, Lb)
            if nargin>0
                this.JAsteel=JAsteel;
                this.S=S;
                this.w1=w1;
                this.w2=w2;
                this.l=l;

                this.R2=R2;
                this.Rb=Rb;
                this.L2=L2;
                this.Lb=Lb;
                
            end            
        end
        
        function val=FUN(this, t_prev, t_curr, i_prev, i_curr, HL_prev, HL_curr, Psi)
            
            H_prev=HL_prev/this.l;
            H_curr=HL_curr/this.l;
            u0=4*pi*10^-7;
            
            delta=JA.Delta(H_curr-H_prev);
            M=Psi/(this.w2*this.S*u0);
            dPsi_dt=this.w2*this.S*u0*this.JAsteel.JASTEP_d(H_prev,M,delta);
            
            w1=this.w1;
            w2=this.w2;
            L2=this.L2;
            Lb=this.Lb;
            
            R2=this.R2;
            Rb=this.Rb;
            
            val= ( dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) - w1/w2*( (R2+Rb)*i_prev+ (L2+Lb)*(i_curr-i_prev)/(t_curr-t_prev)  )  + HL_prev*(R2+Rb)/w2;
            
            %val= ( dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) - w1/w2*( (R2+Rb)*i_curr+ (L2+Lb)*(i_curr-i_prev)/(t_curr-t_prev)  )  + HL_curr*(R2+Rb)/w2;
        end
        
        function val=FUN_unhyst(this, t_prev, t_curr, i_prev, i_curr, HL_prev, HL_curr, Psi)
            u0=4*pi*10^-7;
            H_prev=HL_prev/this.l;
            %H_curr=HL_curr/this.l;
            delta=1;%   1,    
            M=Psi/(this.w2*this.S*u0);
            dPsi_dt=this.w2*this.S*u0*this.JAsteel.JASTEP_unhyst(H_prev,M,delta);
            
            w1=this.w1;
            w2=this.w2;
            L2=this.L2;
            Lb=this.Lb;
            
            R2=this.R2;
            Rb=this.Rb;
            
            val= ( dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) - w1/w2*( (R2+Rb)*i_prev+ (L2+Lb)*(i_curr-i_prev)/(t_curr-t_prev)  )  + HL_prev*(R2+Rb)/w2;
        end
        
        function HL_curr=SearchRoot_NEWTON_unhyst(this, t_prev, t_curr, i1_prev, i1_curr, HL_prev, Psi)
            dir=1;
            dHLmax=0.0001;
            HL1=HL_prev;
            dHLdiff=dHLmax*dir;
            
            for i=1:5
                val1=this.FUN_unhyst(t_prev, t_curr, i1_prev, i1_curr, HL_prev, HL1, Psi);
                val2=this.FUN_unhyst(t_prev, t_curr, i1_prev, i1_curr, HL_prev, HL1+dHLdiff, Psi);
                dval_dH=(val2-val1)/dHLdiff;
                if (dval_dH~=0)
                    HL2=HL1-val1/dval_dH;
                    dHL=HL2-HL1;
                    if abs(dHL)<abs(dHLdiff)&&dHL~=0
                        dHLdiff=abs(dHL)*dir;
                    end
                    HL1=HL2;    
                else
                    break
                end
            end
            
            HL_curr=HL1;
        end
                  

        function HL_curr=FUN_discharge(this, t_prev, t_curr, i1_prev, i1_curr, HL_prev)
            w1=this.w1;
            w2=this.w2;
            L2=this.L2;
            Lb=this.Lb;
            
            R2=this.R2;
            Rb=this.Rb;
            
            
            %0=(dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) - w1/w2*( (R2+Rb)*i1_prev+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  + HL_prev*(R2+Rb)/w2;
            %(dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) = w1/w2*( (R2+Rb)*i1_prev+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_prev*(R2+Rb)/w2;
            %(dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev) = (t_curr-t_prev)*( w1/w2*( (R2+Rb)*i1_prev+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_prev*(R2+Rb)/w2 );
            %(HL_curr-HL_prev) = (dPsi_dt+(L2+Lb)/w2 )^-1 * (t_curr-t_prev)*(w1/w2*( (R2+Rb)*i1_prev+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_prev*(R2+Rb)/w2 );
            %HL_curr = (dPsi_dt+(L2+Lb)/w2 )^-1 *(t_curr-t_prev)*(w1/w2*( (R2+Rb)*i_prev+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_prev*(R2+Rb)/w2 ) +HL_prev;
            
            
            %dPsi_dt =0, 
            
            HL_curr = ((L2+Lb)/w2 )^-1 *(t_curr-t_prev)*(w1/w2*((R2+Rb)*i1_prev+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_prev*(R2+Rb)/w2 ) +HL_prev;
            
            %%%val=w2*(t1-t2)/(L2+Lb)* (  HL_prev*(R2+Rb)/w2 + HL1*(L2+Lb)/w2/(t1-t2)  +w1/w2* ( i_1*(R2+Rb)  +  (i_2-I_1)/(t2-t1)*(L2+Lb) )  );
        end
        
        function HLroot=SearchRoot(this, t_prev, t_curr, i1_prev, i1_curr, HL_prev, Psi)
            [finded1, HLroot1]=this.SearchRoot_NEWTON(t_prev, t_curr, i1_prev, i1_curr, HL_prev, Psi,  1);
            [finded2, HLroot2]=this.SearchRoot_NEWTON(t_prev, t_curr, i1_prev, i1_curr, HL_prev, Psi, -1);         
            
            if  finded1&&~finded2
                HLroot=HLroot1;
            elseif  ~finded1&&finded2 
                HLroot=HLroot2;
            elseif finded1&&finded2
                if abs(HLroot1-HL_prev)<=abs(HLroot2-HL_prev)
                    HLroot=HLroot1;
                else
                    HLroot=HLroot2;
                end
            else
                %HLroot=HL;
                HLroot=this.FUN_discharge(t_prev, t_curr, i1_prev, i1_curr, HL_prev);
                %HLroot=this.SearchRoot_NEWTON_unhyst(t_prev, t_curr, i1_prev, i1_curr, HL_prev, Psi);
            end
        end
                    
        function [finded, HL_curr]=SearchRoot_NEWTON(this, t_prev, t_curr, i1_prev, i1_curr, HL_prev, Psi,dir)
            dHLmax=0.0001;
            HL1=HL_prev;
            dHLdiff=dHLmax*dir;
            
            for i=1:5
                val1=this.FUN(t_prev, t_curr, i1_prev, i1_curr, HL_prev, HL1, Psi);
                val2=this.FUN(t_prev, t_curr, i1_prev, i1_curr, HL_prev, HL1+dHLdiff, Psi);
                dval_dH=(val2-val1)/dHLdiff;
                if (dval_dH~=0)
                    HL2=HL1-val1/dval_dH;
                    if (HL2-HL_prev)*dir<0
                       HL1=HL2;
                       break
                    end
                    dHL=HL2-HL1;
                    if abs(dHL)<abs(dHLdiff)&&dHL~=0
                        dHLdiff=abs(dHL)*dir;
                    end
                    HL1=HL2;    
                else
                    break
                end
            end
            
            HL_curr=HL1;
            if (HL_curr-HL_prev)*dir>=0
                finded=true;
            else
                finded=false;
            end
            
        end
                  
        
        function [finded, HLroot]=SearchRoot_SEQ(this, t_prev, t_curr, i1_prev, i1_curr, HL_curr, Psi,dir)
            %    
            HLmax=100*abs(i1_prev-i1_curr)*this.w1;
            dHL=dir*HLmax/(this.Nstep-1);
            
            finded=false;
            HLroot=NaN;
            
            HL_next=HL_curr;
            val1=this.FUN(t_prev, t_curr, i1_prev, i1_curr, HL_curr, HL_next, Psi);
            % for i=1:this.Nstep
            k=1;
            while true    
                HL_next=HL_next+dHL*k^2;
                val2=this.FUN(t_prev, t_curr, i1_prev, i1_curr, HL_curr, HL_next, Psi);  
                if (val2*val1<=0)
                    finded=true;
                    break
                else
                    val1=val2;
                end
                k=k+1;
                if (k==100)
                    break
                end
            end
            
            if finded==true
                %   
                HL1=HL_next-dHL;
                HL2=HL_next;
                while (true)
                    HL3=(HL1+HL2)/2;
                    val3=this.FUN(t_prev, t_curr, i1_prev, i1_curr, HL_curr, HL3, Psi);
                    if val3*val1<=0 %     
                        HL2=HL3;
                    else %  
                        HL1=HL3;
                    end                    
                    if abs(HL2-HL1)<=this.epsilon*abs(max([HL1 HL2]));
                        break
                    end
                end
                HLroot=(HL1+HL2)/2;
            end
        end
        
        function val=DH1(this, t_prev, t_curr, i1_prev, i1_curr, HL, Psi)
            val= (this.SearchRoot(t_prev, t_curr, i1_prev, i1_curr, HL, Psi)-HL)/(t_curr-t_prev);
        end
        
        function val=DH2(this, t_prev, t_curr, i1_prev, i1_curr, HL, Psi)
            u0=4*pi*10^-7;
            dPsi=(this.w2*this.S*u0)*this.JAsteel.JASTEP_d(HL/this.l, Psi/(this.w2*this.S*u0),1);
            
            
            w1=this.w1;
            w2=this.w2;
            L2=this.L2;
            Lb=this.Lb;
            R2=this.R2;
            Rb=this.Rb;
            
            val= ( (dPsi+(this.L2+this.Lb)/w2)^-1 )*( w1/w2*( (R2+Rb)*i1_prev   +  (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL*(R2+Rb)/w2  );
            
        end
        
        function [HL, Psi, i2_array]=JACalc(this, t_array, i1_array, i2_init, Psi_init)
            u0=4*pi*10^-7;
            
            HL(1)=i2_init*this.w2+i1_array(1)*this.w1;
            i2_array(1)=i2_init;
            Psi(1)=Psi_init;
            
            for k=2:length(t_array)
                dt=t_array(k)-t_array(k-1);
                
               % DH1=this.DH1(t_array(k-1), t_array(k), i1_array(k-1), i1_array(k), HL(k-1), Psi(k-1));
                %DH2=this.DH2(t_array(k-1), t_array(k), i1_array(k-1), i1_array(k), HL(k-1), Psi(k-1));
                HL(k)=HL(k-1)+dt*this.DH1(t_array(k-1), t_array(k), i1_array(k-1), i1_array(k), HL(k-1), Psi(k-1));
                %=HL(k-1)+dHL;
                %disp(k)
                dHL=HL(k)-HL(k-1);
                %Psi(k)=Psi(k-1)+(this.w2*this.S*u0)*this.JAsteel.JASTEP_d(H,M,delta)*dHL/this.l;
                
                Psi(k)=this.GetPsi(HL(k),dHL,Psi(k-1));
                
                i2_array(k)= ( HL(k)-i1_array(k)*this.w1 )/this.w2;
            end
        end
        
        
        function val=GetPsi(this,HL,dHL,Psi_prev)
                u0=4*pi*10^-7;
                delta=JA.Delta(dHL);                
                M=Psi_prev/(this.w2*this.S*u0);
                H=HL/this.l;            
                
                Psi_s=this.JAsteel.Ms*(this.w2*this.S*u0);
                Psi_curr=Psi_prev+(this.w2*this.S*u0)*this.JAsteel.JASTEP_d(H,M,delta)*dHL/this.l;
                
                % ,      .
                if (Psi_curr>=Psi_s) 
                    val=Psi_s;
                elseif (Psi_curr<=-Psi_s) 
                    val=-Psi_s;
                else
                    val=Psi_curr;
                end
        end
        
    end
    

    
    
end

