classdef CT_model2<handle
    %CT_MODEL Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        
        u0=4*pi*10^-7;
        JAsteel %
        S %
        w1 % . 
        w2 % . 
        l %  
        
        R2 % 
        Rb % 
        L2 % 
        Lb  %     
               
        Nstep=100;
        epsilon=0.01
        
        HLprev;
        
        Current % 
        
        dHLmax=10;%    h 
        
        adaptive_step=false;
        
        is_saturated=false;
        
        
    end
    
    methods
        
        function this=CT_model2(JAsteel, S, w1, w2, l, R2, Rb, L2, Lb, Current)
            if nargin>0
                this.JAsteel=JAsteel;
                this.S=S;
                this.w1=w1;
                this.w2=w2;
                this.l=l;

                this.R2=R2;
                this.Rb=Rb;
                this.L2=L2;
                this.Lb=Lb;        
        
                this.Current=Current; %   ,   
            end            
        end
        %  ,    HL_curr,      
        function val = FUN(this, t_prev, t_curr, HL_prev, HL_curr, Psi)   
            
            H_prev=HL_prev/this.l;
            H_curr=HL_curr/this.l;
            
            delta=JA.Delta(H_curr-H_prev);
            M=Psi/(this.w2*this.S*this.u0);
            dPsi_dt=this.w2*this.S*this.u0*this.JAsteel.JASTEP_d(H_curr,M,delta);
            
            w1=this.w1;
            w2=this.w2;
            L2=this.L2;
            Lb=this.Lb;
            
            R2=this.R2;
            Rb=this.Rb;
            
            i1=this.Current.i(t_curr);
            di1_dt=this.Current.di(t_curr);
            
            val= ( dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) - w1/w2*( (R2+Rb)*i1 + (L2+Lb)*di1_dt  )  + HL_curr*(R2+Rb)/w2;
        end
        
        function [finded, HL_curr] = SearchRoot_NEWTON(this, t_prev, t_curr, HL_prev, Psi,dir)
            dHLstep=0.000001;
            HL1=HL_prev;
            dHLdiff=dHLstep*dir;
            
            for i=1:5
                val1=this.FUN(t_prev, t_curr, HL_prev, HL1, Psi);
                val2=this.FUN(t_prev, t_curr, HL_prev, HL1+dHLdiff, Psi);
                dval_dH=(val2-val1)/dHLdiff;
                if (dval_dH~=0)
                    HL2=HL1-val1/dval_dH;
                    if (HL2-HL_prev)*dir<0
                       HL1=HL2;
                       break
                    end
                    dHL=HL2-HL1;
                    if abs(dHL)<abs(dHLdiff)&&dHL~=0
                        dHLdiff=abs(dHL)*dir;
                    end
                    HL1=HL2;    
                else
                    break
                end
            end
            
            HL_curr=HL1;
            if (HL_curr-HL_prev)*dir>=0
                finded=true;
            else
                finded=false;
            end
            
        end

        function HLroot = SearchRoot(this, t_prev, t_curr, HL_prev, Psi)
            [finded1, HLroot1]=this.SearchRoot_NEWTON(t_prev, t_curr, HL_prev, Psi,  1);
            [finded2, HLroot2]=this.SearchRoot_NEWTON(t_prev, t_curr, HL_prev, Psi, -1);         
            
            if  finded1&&~finded2
                HLroot=HLroot1;
            elseif  ~finded1&&finded2 
                HLroot=HLroot2;
            elseif finded1&&finded2
                if abs(HLroot1-HL_prev)<=abs(HLroot2-HL_prev)
                    HLroot=HLroot1;
                else
                    HLroot=HLroot2;
                end
            else
                %HLroot=HL;
                HLroot=this.FUN_discharge(t_prev, t_curr, HL_prev);
                %HLroot=this.SearchRoot_NEWTON_unhyst(t_prev, t_curr, i1_prev, i1_curr, HL_prev, Psi);
            end
            
        end      

        function HL_curr = FUN_discharge(this, t_prev, t_curr, HL_prev)
            w1=this.w1;
            w2=this.w2;
            L2=this.L2;
            Lb=this.Lb;
            
            R2=this.R2;
            Rb=this.Rb;
            
            %0=(dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) - w1/w2*( (R2+Rb)*i1_curr+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  + HL_curr*(R2+Rb)/w2;   
            %(dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev)/(t_curr-t_prev) = w1/w2*( (R2+Rb)*i1_curr+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_curr*(R2+Rb)/w2;
            %(dPsi_dt+(L2+Lb)/w2 ) * (HL_curr-HL_prev) = (t_curr-t_prev)*( w1/w2*( (R2+Rb)*i1_curr+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_curr*(R2+Rb)/w2 );
            %(HL_curr-HL_prev) = (dPsi_dt+(L2+Lb)/w2 )^-1 * (t_curr-t_prev)*(w1/w2*( (R2+Rb)*i1_curr+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_curr*(R2+Rb)/w2 );   
            %HL_curr = (dPsi_dt+(L2+Lb)/w2 )^-1 *(t_curr-t_prev)*(w1/w2*( (R2+Rb)*i_curr+ (L2+Lb)*(i1_curr-i1_prev)/(t_curr-t_prev)  )  - HL_curr*(R2+Rb)/w2 ) +HL_prev;
            
            %dPsi_dt =0, 
            i1=this.Current.i(t_prev);
            di1_dt=this.Current.di(t_prev);
            
            HL_curr = ((L2+Lb)/w2 )^-1 *(t_curr-t_prev)*(w1/w2*((R2+Rb)*i1 + (L2+Lb)*di1_dt  )  - HL_prev*(R2+Rb)/w2 ) +HL_prev;
            
        end
       
        function [dHL, dt] = DH(this, t_prev, dt_max, HL_prev, Psi)
            %   ,   dt
            dt=dt_max;
            k=0;
            while true
                t_curr = t_prev + dt;
                HL_curr=this.SearchRoot(t_prev, t_curr, HL_prev, Psi);
                dHLstep=HL_curr-HL_prev;
            
                if this.adaptive_step==false
                    break
                end                
                if abs(dHLstep)<=this.dHLmax
                    break
                end
                dt=dt*0.5;
                k=k+1;
                if k==10
                    break;
                end
            end
            
            dHL=HL_curr-HL_prev;
        end

        function [HL, Psi, i1, i2, t] = JACalc(this, Psi_init, i2_init, t_start, t_end, dt_max)
           i1_init=this.Current.i(t_start);
           HL_init=i1_init*this.w1+i2_init*this.w2;
           
           %M_init=Psi_init/(this.u0*this.w2*this.S);
           %   
           k=1;
           i1(1) = i1_init;
           i2(1) = i2_init;
           Psi(1) = Psi_init;
           HL(1) = HL_init;
           t(1) = t_start;
           
           step=1;
           while true
               [dHL,dt] = this.DH(t(k), dt_max, HL(k), Psi(k));%     ,  dHL     
               
               HL(k+1) = HL(k)+dHL;

               %Psi(k+1) = this.GetPsiCycle(HL(k), HL(k+1), Psi(k));
               Psi(k+1) = this.GetPsiCycle(HL(k), HL(k+1), Psi(k));
               t(k+1) = t(k)+dt;
               i1(k+1) = this.Current.i(t(k+1));
               i2(k+1) = ( HL(k+1) - i1(k+1) ) / this.w2;
               
               if t(k+1)>=t_end
                   break
               end
               
               if step==1000
                   step= 1;
                   disp(t(k))
               end
               
               k=k+1;
               step=step+1;
           end
           
        end
        
        function [HL, Psi, i1, i2, t] = JACalc_an(this, Psi_init, i2_init, t_start, t_end, dt_max)
           hyst_status=this.JAsteel.hyst_enable; %    
           
           i1_init=this.Current.i(t_start);
           %HL_init=i1_init*this.w1+i2_init*this.w2;
           
           HL_init=this.GetStartHL(Psi_init);
           this.JAsteel.hyst_enable=false;
           
           %M_init=Psi_init/(this.u0*this.w2*this.S);
           %   
           k=1;
           i1(1) = i1_init;
           i2(1) = i2_init;
           Psi(1) = Psi_init;
           HL(1) = HL_init;
           t(1) = t_start;
           
           step=1;
           while true
               [dHL,dt] = this.DH(t(k), dt_max, HL(k), Psi(k));%     ,  dHL     
               
               HL(k+1) = HL(k)+dHL;

               %Psi(k+1) = this.GetPsiCycle(HL(k), HL(k+1), Psi(k));
               Psi(k+1) = this.GetPsiCycle(HL(k), HL(k+1), Psi(k));
               t(k+1) = t(k)+dt;
               i1(k+1) = this.Current.i(t(k+1));
               i2(k+1) = ( HL(k+1) - i1(k+1) ) / this.w2;
               
               if t(k+1)>=t_end
                   break
               end
               
               if step==1000
                   step= 1;
                   disp(t(k))
               end
               
               k=k+1;
               step=step+1;
           end
           
           this.JAsteel.hyst_enable=hyst_status;
        end
        
        function val=GetPsiCycle(this, HL1, HL2, Psi_prev) %  -  
            M_init=Psi_prev/(this.w2*this.S*this.u0);    
            dHL=HL2-HL1;
            N=ceil(dHL/this.dHLmax);
            
            if N<=20
                N=21;
            end
            
            H1=HL1/this.l;% 
            H2=HL2/this.l;
            
            if abs(H1)>=this.JAsteel.H_sat1&&abs(H2)>=this.JAsteel.H_sat1&&this.is_saturated==false
                this.is_saturated=true;
            elseif abs(H1)<=this.JAsteel.H_sat2&&abs(H2)<=this.JAsteel.H_sat2&&this.is_saturated==true
                this.is_saturated=false;
            end
            
            M=this.JAsteel.GetMCycle(H1, H2, N, M_init, this.is_saturated);
            val=this.w2*this.S*this.u0*M;
        end
        
        
        function HL=GetStartHL(this, Psi_init)
            HL=0;
            for i=1:5
                HL=HL- (this.dPsi_dH_an(HL)^-1)*(this.Psi_an(HL)-Psi_init);
            end
        end
        
        function val=Psi_an(this, HL)
            Ms=this.JAsteel.Ms;
            a=this.JAsteel.a;
            alpha=0;
            M=0;
            H=HL/this.l;
            
            val=(this.w2*this.S*this.u0)*JA.Man(H,M,a,alpha,Ms);
        end
        
        function val=dPsi_dH_an(this, HL)
            Ms=this.JAsteel.Ms;
            a=this.JAsteel.a;
            alpha=0;
            M=0;
            H=HL/this.l;
            
            val=(this.w2*this.S*this.u0)*JA.dMan_dH(H,M,a,alpha,Ms);
        end
        
    end        
        
        
        
end
    

    
    


