classdef JA<handle
    %JA Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        Ms
        a
        %a_an %  ,  a,  
        alpha
        k
        c
        
        H_sat1 %,          .
        H_sat2 %,          .
        
        hyst_enable=true;
        
        saturation_anhyst=false
        stabilizeM=false
    end
    
    
    methods
        
        function this=JA(Ms,a, alpha,k,c)
            if nargin>0
                this.Ms=Ms;
                this.a=a;
                %this.a_an=a_an;
                this.alpha=alpha;
                this.k=k;
                this.c=c;
                
                this.H_sat1=24*a; %    
                this.H_sat2=36*a; %     
            end            
        end
                
        function val=JASTEP_d(this,H,M,delta)
            val=JA.DM_dH(H, M, delta, this.Ms, this.a, this.alpha, this.c, this.k, this.hyst_enable);  
        end
        
        function val=JASTEP_anhyst(this,H,M,delta)
            val=JA.DM_dH(H,M,delta, this.Ms, this.a, this.alpha, this.c, this.k , false);  
        end
        
        function M=M(this, H, M_init) %
            M(1)=M_init;                    
            for i=2:length(H)
                dH=H(i)-H(i-1);
                if dH>=0
                    delta=1;
                else
                    delta=-1;
                end
                q1=this.JASTEP_d(H(i-1), M(i-1), delta);
                q2=this.JASTEP_d(H(i-1)+dH/2, M(i-1)+dH/2*q1, delta);
                q3=this.JASTEP_d(H(i-1)+dH/2, M(i-1)+dH/2*q2, delta);
                q4=this.JASTEP_d(H(i-1)+dH, M(i-1)+dH*q3, delta);

                M(i) = M(i-1) + dH/6*(q1 + 2*q2 + 2*q3 + q4);
            end
        end
        
        function M=M_anhyst(this, H, M_init) %
            M(1)=M_init;                    
            for i=2:length(H)
                dH=H(i)-H(i-1);
                if dH>=0
                    delta=1;
                else
                    delta=-1;
                end
                q1=this.JASTEP_anhyst(H(i-1), M(i-1), delta);
                q2=this.JASTEP_anhyst(H(i-1)+dH/2, M(i-1)+dH/2*q1, delta);
                q3=this.JASTEP_anhyst(H(i-1)+dH/2, M(i-1)+dH/2*q2, delta);
                q4=this.JASTEP_anhyst(H(i-1)+dH, M(i-1)+dH*q3, delta);

                M(i) = M(i-1) + dH/6*(q1 + 2*q2 + 2*q3 + q4);
            end
            
        end
        
        %   ,  " ".( )
        function M=Msingle(this, H, M_init) %
            M=M_init;                    
            for i=2:length(H)
                dH=H(i)-H(i-1);
                if dH>=0
                    delta=1;
                else
                    delta=-1;
                end
                Hval=H(i);
                q1=this.JASTEP_d(Hval, M, delta);
                q2=this.JASTEP_d(Hval+dH/2, M+dH/2*q1, delta);
                q3=this.JASTEP_d(Hval+dH/2, M+dH/2*q2, delta);
                q4=this.JASTEP_d(Hval+dH, M+dH*q3, delta);

                M = M + dH/6*(q1 + 2*q2 + 2*q3 + q4);
            end
        end
        
        function M=Msingle_anhyst(this, H, M_init) %
            M=M_init;                    
            for i=2:length(H)
                dH=H(i)-H(i-1);
                if dH>=0
                    delta=1;
                else
                    delta=-1;
                end
                
                Hval=H(i);
                q1=this.JASTEP_anhyst(Hval, M, delta);
                q2=this.JASTEP_anhyst(Hval+dH/2, M+dH/2*q1, delta);
                q3=this.JASTEP_anhyst(Hval+dH/2, M+dH/2*q2, delta);
                q4=this.JASTEP_anhyst(Hval+dH, M+dH*q3, delta);

                M = M + dH/6*(q1 + 2*q2 + 2*q3 + q4);
            end
            
        end
        
        function val=GetMCycle(this, H1, H2, N, M_init, is_saturated)           
            dH=(H2-H1)/(N-1);
            H=H1:dH:H2;
            
            if this.saturation_anhyst==true&&is_saturated %       
               M=this.Msingle_anhyst(H, M_init);    
               val=M;
            else
               M=this.Msingle(H, M_init);%-  
               Mval=M;
               %
               if this.stabilizeM==true&&Mval>=this.Ms 
                  val=this.Ms;
               elseif this.stabilizeM==true&&Mval<=-this.Ms
                  val=-this.Ms;
               else
                  val=Mval;
               end
            end
            
        end

    end

    methods (Static)
        
        function val=Man(H,M,a,alpha,Ms)
            if (H==0&&M==0)
                val=0;
            else
                val=Ms*(coth((H+alpha*M)/a) - a/(H+alpha*M));
            end
        end     
        
        function val=dMan_dH(H,M,a,alpha,Ms)
            if (abs(H+alpha*M)<=1e-6)%!
                val=Ms/(3*a);
            else
                val=-Ms*(  (coth((H+alpha*M)/a)^2-1)/a - a/(H+alpha*M)^2);
            end
        end    
        
        function val=DM_dH(H,M,delta,Ms,a,alpha,c,k, hyst_enable)
            
            if hyst_enable
                dM=JA.Man(H,M,a,alpha,Ms)-M;
                val=1/(1+c)*dM/(k*delta-alpha*dM) + c/(1+c)*JA.dMan_dH(H,M,a,alpha,Ms);
            else
                alpha=0;
                val=JA.dMan_dH(H,M,a,alpha,Ms);
            end
        end
        
        function val=Delta(dH)
                if dH>=0
                    val=1;
                else
                    val=-1;
                end
        end
        
    end
    
end

